/*
Copyright (C) 2003  Francesco Stablum "DuskDruid"

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "myheaders.h"
#define MAXCONN 16384
#define MAXMALLOC 16777216
/*extern declarations*/

extern struct flags_t flags;

int display_status(FILE * out,struct conn_t * conn,int status)
{
  char *client_name;
  char *server_name;
  char *s_time=(char *)S_calloc(128,1);
  char *status_string=(char *)S_calloc(1,128);
  client_name=(char *)cache_getnamebyip(conn->client_ip);
  server_name=(char *)cache_getnamebyip(conn->server_ip);

  if (!flags.silent)
    {

      switch(status)
	{
	case 1:
	  {
	    strcpy(status_string,"SYN-SENT");
	    break;
	  }
	case 2:
	  {
	    strcpy(status_string,"SYN-RECEIVED");
	    break;
	  }
	case 3:
	  {
	    strcpy(status_string,"ESTABLISHED");
	    break;
	  }
	case 4:
	  {
	    strcpy(status_string,"FIN-WAIT-1");
	    break;
	  }
	case 5:
	  {
	    strcpy(status_string,"FIN-WAIT-2");
	    /* should it be CLOSE-WAIT? */
	    break;
	  }
	case 6:
	  {
	    strcpy(status_string,"TIME-WAIT");
	    /* should it be LAST-ACK? */
	    break;
	  }
	case 7:
	  {
	    strcpy(status_string,"CLOSED");
	    /* should it be TIME-WAIT? */
	    break;
	  }
	default:
	  {
	    strcpy(status_string,"UNDER-DEVELOPMENT");
	    break;
	  }
	}
      get_numbered_time(s_time);
      if(out==stdout)
	timecolor();
      fprintf (out,"%-16s ",s_time);
      if(out==stdout)
	numberingcolor();
      fprintf (out,"%-6d ",conn->conn_number);
      allstatusescolor(out,status);
      fprintf(out,"%-14s",status_string);
      if(out==stdout)
	resetcolor();
      
      S_free(status_string);

      if(out==stdout)
	clientnamecolor();
      fprintf(out," %s:%s ",
	      client_name,
	      getportname(conn->client_port)
	      );
      if(out==stdout)
	resetcolor();

      allstatusescolor(out,status);
      fprintf(out,"> ");
      if(out==stdout)
	resetcolor();

      if(out==stdout)
	servernamecolor();
      fprintf(out,"%s",
	      server_name
	      );
      if(out==stdout)
	resetcolor();

      fprintf(out,":");

      if(out==stdout)
	servicecolor();
      fprintf(out,"%s",
	      getportname(conn->server_port)
	      );
      if(out==stdout)
	resetcolor();
      
      fprintf(out,"\n");
      
    }

}
int display_header( FILE * out, struct ip *ippacket, struct tcphdr *tcppacket)
{
  char * source=NULL;
  char * dest=NULL;
  char * s_time=(char *)S_calloc(128,1);
  source=(char *)cache_getnamebyip(ippacket->ip_src.s_addr);
  dprintf("display_header(): source(%x)",source);
  dprintf("display_header(): source=%s",source);
  get_numbered_time(s_time);
  if(out==stdout)
    timeheadercolor();
  fprintf (out,"%-16s ",s_time);
  if(out==stdout)
    clientnameheadercolor();
  fprintf(out,"%s",source);
  if(out==stdout)
    resetcolor();
  
  fprintf(out,":");

  if(out==stdout)
    serviceheadercolor();
  fprintf(out,"%s ",getportname(tcppacket->source));
  if(out==stdout)
    resetcolor();

  if(out==stdout)
    flagscolor();
  if(tcppacket->urg)
    fprintf(out,"U");
  if(tcppacket->ack)
    fprintf(out,"A");
  if(tcppacket->psh)
    fprintf(out,"P");
  if(tcppacket->rst)
    fprintf(out,"R");
  if(tcppacket->syn)
    fprintf(out,"S");
  if(tcppacket->fin)
    fprintf(out,"F");
  if(out==stdout)
    resetcolor();
  
  dest=(char *)cache_getnamebyip(ippacket->ip_dst.s_addr);

  if(out==stdout)
    directioncolor();
  fprintf(out," > ");
  if(out==stdout)
    resetcolor();

  if(out==stdout)
    servernameheadercolor();
  fprintf(out,"%s",dest);
  if(out==stdout)
    resetcolor();

  fprintf(out,":");

  if(out==stdout)
    serviceheadercolor();
  fprintf(out,"%s\n",getportname(tcppacket->dest));
  if(out==stdout)
    resetcolor();

  /* dacanc     fprintf(out,"[seq:%u] [ack:%u]\n",ntohl(tcppacket->seq),ntohl(tcppacket->ack_seq));*/
  fflush(out);

}

int display_data(FILE * out,u_char * payload,int payload_len, int raw_flag,int lf_flag,int hexnonprint_flag)
{
  int curr_pos;
  u_char * curr_char=payload;
  for(curr_pos=0; curr_pos<payload_len;curr_pos++)
    {
      if(raw_flag)
	fprintf(out,"%c",*curr_char); /* FIXME: too slow!*/
      else
	if(flags.displayhex)
	  fprintf(out,"%2.2x ",*curr_char); 
	else
	  {
	    if (isprint(*curr_char)==0)
	      {

		//		  fprintf(out,"NONPRINT\n");
		if (*curr_char=='\n')
		  fprintf(out,"\n");
		else
		  if (*curr_char!=13)
		    {
		      if(out==stdout)
			nonprintcolor();
		      if(hexnonprint_flag)
			fprintf(out,"<%x>",*curr_char);
		      else
			{
			  *curr_char='.';
			  fprintf(out,"%c",*curr_char); /* FIXME: too slow! */
			}
		      if(out==stdout)
			payloadcolor();
		    }
	      }
	    else
	      fprintf(out,"%c",*curr_char);
	  }
      curr_char++;
    }

  
  if(flags.displayhex)
    {
      if ( lf_flag&&
	   (curr_pos>0)
	   )
	fprintf(out,"\n");
    }
    else
      if ( lf_flag&&
	   (curr_pos>0)&&
	   *(curr_char-1)!='\n'
	   )
	fprintf(out,"\n");
  fflush(out);
}
int display_dump(FILE * out,u_char * payload, int payload_len, int ascii_flag, int linenum)
{
  int curr_pos=0;
  int spacesleft=0;
  int count_spaces=0;
  u_char * curr_char=payload;
  //  u_char * curr_beg=payload;
  u_char ascii_string[17]="..............";
  if(payload_len>0)
    {
      for(curr_pos=0;curr_pos<payload_len;curr_pos++)
	{
	  if(curr_pos==0)
	    {
	      if(out==stdout)
		firsthexcountcolor();
	      fprintf(out,"0x0000  ",curr_pos);
	      if(out==stdout)
		resetcolor();
	    }
	  else 
	    if((curr_pos)%16==0) /* print ascii, newline */
	      {
		if(out==stdout)
		  hexcountcolor();
		fprintf(out,"%-#8.4x",curr_pos);
		if(out==stdout)
		  resetcolor();
	      }
	  if(isprint(*curr_char)==0||
	     *curr_char==' ')
	    ascii_string[curr_pos%16]='.';
	  else
	    ascii_string[curr_pos%16]=*curr_char;
	  if(isprint(*(curr_char+1))==0||
	     *(curr_char+1)==' ')
	    ascii_string[(curr_pos%16)+1]='.';
	  else
	    ascii_string[(curr_pos%16)+1]=*(curr_char+1);
	  fprintf(out," %2.2x%2.2x",*curr_char,*(curr_char+1));
	  if((curr_pos+2)%16==0) /* print ascii, newline */
	    {
	      ascii_string[16]='\0';
	      if(ascii_flag)
		{
		  if(out==stdout)
		    hexasciicolor();
		  fprintf(out,"        %s\n",ascii_string);
		  if(out==stdout)
		    resetcolor();

		}
	      else
		fprintf(out,"\n");
	    }
	  curr_char+=2;
	  curr_pos++; /* we need 2 times curr_pos++ 
			 and curr_char++ */
	}
      if(curr_pos%16!=0)
	if(ascii_flag==1)
	  {
	    spacesleft=(16-(curr_pos%16))*2+8+(16-(curr_pos%16))/2; //+(16-(curr_pos%16)%2);
	    for (count_spaces=0;count_spaces<spacesleft;count_spaces++)
	      fprintf(out," ");
	    //	  fprintf(out,"%d:%d:%d:%d\n%",spacesleft,curr_pos,curr_pos%16,(16-(curr_pos%16))/2);
	    ascii_string[curr_pos%16]='\0';
	    if(out==stdout)
	      hexasciicolor();
	    fprintf(out,"%s\n",ascii_string);
	    if(out==stdout)
	      resetcolor();
	  }
	else
	  {
	    fprintf(out,"\n");
	  }
      fflush(out);
    }
}
