/*
Copyright (C) 2003  Francesco Stablum "DuskDruid"
<duskdruid @ inventati org>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "myheaders.h"

/*extern declarations*/
extern struct flags_t flags;
/*structure for tracking a connection*/

struct conn_t * first_conn=NULL; /*all connections, ends on a 0*/
struct conn_t * last_conn=NULL; 
int conn_num; /* number of active connections */


int * seek_zero(int * ptr)
{
  int count=0; 
   
  while(*ptr&&(count<MAXCONN))
    {
      ptr++;
      count++;
    }
  if(flags.debug)
    printf("*d* seek_zero(): found free pointer at %x\n",ptr);
  return ptr;  
}
int status_switch(struct conn_t * conn,int status)
{
  set_tracked_color(conn->conn_number);
  if(!flags.silent)
    display_status(stdout,conn,status);
  if(flags.writestatus)
    {
      if(!(conn->file_opened))
	open_file(conn);
      display_status(conn->client_file,conn,status);
      display_status(conn->server_file,conn,status);
    }

  conn->status=status;
}

int newconn(struct conn_t * prev_ring,
	    struct in_addr client, 
	    struct in_addr server,
	    u_int32_t seq,
	    u_int16_t client_port,
	    u_int16_t server_port)
{
  struct conn_t* conn;
  conn_num++;
  dprintf("[newconn] new connection(%d):",conn_num);
  dprintf("[newconn] seq: %u",seq);
  dprintf("[newconn] src: %s:%u dst: %s:%u",
	   inet_ntoa(client),
	   ntohs(client_port),
	   inet_ntoa(server),
	   ntohs(server_port)
	   );
  dprintf("[newconn] newtrack(): sizeof(struct conn_t)=%d, conn=%x\n", sizeof(struct conn_t),conn);

  conn=(struct conn_t *)S_calloc(sizeof(struct conn_t),1);
  prev_ring->next_conn=conn; /* and the chain becomes longer... :)
				thank you mainman for this good idea! */
  last_conn=conn;
  dprintf("[newconn] prev_ring->next_conn=conn=%x\n", conn);

  (conn)->client_ip=client.s_addr;
  (conn)->server_ip=server.s_addr;
  (conn)->client_port=client_port;
  (conn)->server_port=server_port;
  (conn)->client_seq=seq;
  (conn)->conn_number=conn_num;
  status_switch(conn,1);
}

int data_write(struct conn_t * conn_ptr,
	       u_int32_t data_off, /* offset from beginning */
	       u_char * payload,
	       int payload_len,
	       Part side /* client (1) or server (2) ? */
	       ) /* FIXME: rewrite whole function!!! */
{
  /*******************************************************************
   * this function has the purpose to write the data of the tcp
   * session *ORDERED* to a file. This means *discarding* data not
   * according to the current offset, i.e. duplicates and packets that
   * come to the interface before others that have delayed
   *******************************************************************/

  if(side==CLIENT && conn_ptr->client_data_size==data_off ||
     side==SERVER && conn_ptr->server_data_size==data_off)
    {
      if(flags.file)
	/* is the flag for writing to file active? */
	{
	  if(conn_ptr->file_opened==0)
	    /* if the file is not opened, open it */
	    open_file(conn_ptr);
	  if(!flags.writeheaders
	     &&
	     !flags.writestatus
	     &&
	     !flags.writeoffset)
	    /* if write header/status is NOT selected */
	    {
	      file_write(conn_ptr, payload, payload_len, side); /* write the data */
	    }
	  else	      
	    /* if write header/status is selected */
	    {
	      /* 
		 here should be nothing because not 
		 all packets go to data_write functions, 
		 but only the recognized ones 
		 FIXME
	      */
	    }
	}
    }

  if (payload_len>0)
    if (side==CLIENT)
      /* client data */
      {
	/* calculate size of data stored in the file */
	conn_ptr->client_data_size=payload_len+data_off;
	dprintf("[data_write] conn_ptr(%x)->client_data_size=%d",
		conn_ptr,
		conn_ptr->client_data_size);
      }
    else /* side == SERVER */
       /* server data */
      {
	conn_ptr->server_data_size=data_off+payload_len;
	dprintf("[data_write] conn_ptr(%x)->client_data_size=%d",
		conn_ptr,
		conn_ptr->client_data_size);
      }
}
