/*
Copyright (C) 2003, 2004  Francesco Stablum "DuskDruid"
<duskdruid @ inventati org>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "myheaders.h"

extern struct conn_t * first_conn;
extern struct conn_t * last_conn;

int verify(struct ip *ippacket,
	   struct tcphdr *tcppacket,
	   u_char *payload,
	   int payload_len
	   )
	
{
 struct conn_t * current_conn;
 struct in_addr in_addr_client;
 struct in_addr in_addr_server;
 int count=0;
 u_int32_t data_off=0;
 current_conn=first_conn;

 /*SYN-SENT: (outside of the loop because the loop is used for sessions just tracked)*/	 
 if(match(current_conn,ippacket,tcppacket,
	  -1,-1,
	  0,0,0,0,1,0)
    )
   { /* IT IS SYN-SENT */
     newconn(last_conn,
	     ippacket->ip_src,
	     ippacket->ip_dst, 
	     tcppacket->seq,
	     tcppacket->source,
	     tcppacket->dest
	     ); /* track new connection */
     return(1);
   }

 current_conn=first_conn; 

 while(current_conn!=NULL)
   {
     count++;
     in_addr_client.s_addr=(current_conn)->client_ip;
     in_addr_server.s_addr=(current_conn)->server_ip;
     
     /* main cycle */
     /* writing headers if writeheaders_flag to the file */
     if(match(current_conn,ippacket,tcppacket,
	      3,-1,
	      -1,-1,-1,-1,-1,-1)
	)
       {
	 write_banners_and_data(current_conn,ippacket,tcppacket,payload,payload_len);
       }
     
     
     /* SYN-RECEIVED:*/
     if(match(current_conn,ippacket,tcppacket,
	      2,1,
	      0,1,0,0,1,0)
	)
       {  
	 if (tcppacket->ack_seq==0||
	     tcppacket->seq==0) 
	   {
	     /* FIXME: choose a better xprintf */
	     printf("WARNING: strange syn/ack packet: ack_seq: %u seq:%u",
		    tcppacket->ack_seq,
		    tcppacket->seq);
	   }
	 else
	   /* IT IS SYN-RECEIVED */
	   {
	     status_switch(current_conn,2);
	     (current_conn)->server_seq=tcppacket->seq;
	     dprintf("[verify] found matching connection (%x) in SYN_SENT status:\n",
		     current_conn);
	     dprintf("[verify] client: %s:%u server: %s:%u \n",
		     inet_ntoa(in_addr_client),
		     ntohs((current_conn)->client_port),
		     inet_ntoa(in_addr_server),
		     ntohs((current_conn)->server_port)
		     );
	   }
       }
     
     /* ESTABLISHING... (3rd packet)*/
     if(match(current_conn,ippacket,tcppacket,
	      1,2,
	      0,1,0,0,0,0)
	)
       if(ntohl((current_conn)->client_seq)+1==ntohl(tcppacket->seq)&&
	  ntohl((current_conn)->server_seq)+1==ntohl(tcppacket->ack_seq)
	  )
	 {
	   /*this is really the third packet of the 3wh
	     3wh completed!*/
	   dprintf("[verify] found matching connection (%x) in SYN_RECEIVED status:",
		   current_conn
		   );
	   dprintf("[verify] client: %s:%u server: %s:%u",
		   inet_ntoa(in_addr_client),
		   ntohs((current_conn)->client_port),
		   inet_ntoa(in_addr_server),
		   ntohs((current_conn)->server_port)
		   );
	   dprintf("[verify] status switched to ESTABLISHED");
	   
	   /*switch connection status to "ESTABLISHED"*/
	   status_switch(current_conn,3);
	   return 1;
	 }
       else
	 /*strange seq and ack_seq fields */
	 dprintf("WARNING: seq and ack tells this is not the 3rd packet of three-way handshake!\n");
     
     
     
     
     /* ESTABLISHED: client sends data */
     if(match(current_conn,ippacket,tcppacket,
	      1,3,
	      0,1,-1,0,0,0)
	)
       {
	 data_off=(ntohl(tcppacket->seq)-ntohl((current_conn)->client_seq))-1;
	 dprintf("data_off:%u",data_off);
	 data_write(current_conn,
		    data_off,
		    payload,
		    payload_len,
		    CLIENT
		    ); /* write the data to terminal, memory or file */
	 return 1;
       }
     
     /* ESTABLISHED: server sends data */
     if(match(current_conn,ippacket,tcppacket,
	      2,3,
	      0,1,-1,0,0,0)
	)
       {
	 data_off=(ntohl(tcppacket->seq)-ntohl((current_conn)->server_seq))-1;
	 dprintf("data_off:%u",data_off);
	 data_write(current_conn,
		    data_off,
		    payload,
		    payload_len,
		    SERVER
		    ); /* write the data to terminal, memory or file */
	 return 1;
       }
     
     
     
     
     /* first FIN sent from client*/ 
     if(match(current_conn,ippacket,tcppacket,
	      1,3,
	      0,-1,-1,0,0,1)
	)
       {
	 /* this is a FIN-WAIT-1 status! */
	 status_switch(current_conn,4);
	 (current_conn)->closer=1; /* is the client that closes the connection */
	 return 1;
       }
     
     
     
     /* first FIN sent from server*/ 
     if(match(current_conn,ippacket,tcppacket,
	      2,3,
	      0,-1,-1,0,0,1)
	)
       {
	 /* this is a FIN-WAIT-1 status! */
	 status_switch(current_conn,4);
	 (current_conn)->closer=2; /* is the server that closes the connection */
	 return 1;
       }
     
     
	 
     /* ACK of first FIN sent from client*/ 
     if(match(current_conn,ippacket,tcppacket,
	      1,4,
	      0,1,-1,0,0,0)
	)
       {
	 /* this is a FIN-WAIT-2/CLOSE-WAIT status! */
	 if((current_conn)->closer==2) 
	   /* is the server that closes the connection */
	   {
	     status_switch(current_conn,5);
	     return 1;
	   }
       }
	 
     
     
     /* ACK of first FIN sent from server*/ 
     if(match(current_conn,ippacket,tcppacket,
	      2,4,
	      0,1,-1,0,0,0)
	)
       {
	 /* this is a FIN-WAIT-2/CLOSE-WAIT status! */
	 if((current_conn)->closer==1)
	   /* is the client that closes the connection */
	   {
	     status_switch(current_conn,5);
	     return 1;
	   }
       }
     
	 
     
     
     /* second FIN sent from client */ 
     if(match(current_conn,ippacket,tcppacket,
	      1,4,
	      0,1,-1,0,0,1)
	||
	match(current_conn,ippacket,tcppacket,
	      1,5,
	      0,1,-1,0,0,1)
	)
       {
	 /* this is a TIME-WAIT/LAST-ACK status! */
	 if((current_conn)->closer==2) 
	   /* is the server that closes the connection */
	   {
	     status_switch(current_conn,6);
	     return 1;
	   }
       }
	 
     
     
     /* second FIN sent from server*/ 
     if(match(current_conn,ippacket,tcppacket,
	      2,4,
	      0,1,-1,0,0,1)
	||
	match(current_conn,ippacket,tcppacket,
	      2,5,
	      0,1,-1,0,0,1)
	)
       {
	 /* this is a TIME-WAIT/LAST-ACK status! */
	 if((current_conn)->closer==1)
	   /* is the client that closes the connection */
	   {
	     status_switch(current_conn,6);
	     return 1;
	   }
       }
     
     /* LAST-ACK sent from client */
     if(match(current_conn,ippacket,tcppacket,
	      1,6,
	      0,1,-1,0,0,0)
	)
       {
	 if((current_conn)->closer==1)
	   /* is the server that closes the connection */
	   /* this is a LAST-ACK status! */
	   {
	     status_switch(current_conn,7);
	     return 1;
	   }
       }
     
     /* LAST-ACK sent from server */
     if(match(current_conn,ippacket,tcppacket,
	      2,6,
	      0,1,-1,0,0,0)
	)
       {
	 if((current_conn)->closer==2)
	   /* is the client that closes the connection */
	   /* this is a LAST-ACK status! */
	   {
	     status_switch(current_conn,7);
	     return 1;
	   }
       }
     current_conn=current_conn->next_conn;
   }
} 


